/*
 * Simple fragment shader for calculating output fragment color.
 */

#version 140          // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2

// uniform shader-parameters
uniform vec4 cam_position;
uniform vec3 light_position;

// light components
uniform vec4 diffuse_color;
uniform vec4 specular_color;
uniform vec4 ambient_color;

// material constants for light components
uniform vec4 diffuse_mat;
uniform vec4 specular_mat;
uniform vec4 ambient_mat;

// the exponent for the specular component
uniform float spec_power;

// sampler holding texture data
uniform sampler2D colorTexture;
uniform sampler2DShadow shadow_map;
//uniform sampler2D shadow_map;

// fragment-shader input variables
in vec4 frag_color;
in vec3 world_normal;
in vec4 world_position;
in vec2 fragTexCoord;
in vec4 SM_tex_coord; // passed on from VS

// fragment-shader output variable (-> stored in the frame-buffer, i.e. "the pixel you see")
out vec4 shadow2Color; // final fragment color


void main()
{
	// renormalize and homogenize input variables
	vec3 normal = normalize(world_normal);
    vec3 position = world_position.xyz / world_position.w;
    
    // calculate the light-direction
    vec3 light_dir = normalize(light_position - position);
	
	 // calculate haveway vector
	vec3 view = normalize(cam_position.xyz - position);
	vec3 halfWay = normalize(light_dir + view);
	
	//get texel color
	vec4 texel = texture(colorTexture, fragTexCoord);
	
	// calculate lighting
    vec4 ambient = ambient_color* ambient_mat * texel;
    vec4 diffuse = texel * diffuse_color * diffuse_mat * max(0.0, dot(normal, light_dir));
    
    //calculate specular
    float spec = max(0.0, dot(normal, halfWay));
    float factor = pow(spec, spec_power);
    vec4 specular = specular_color * specular_mat * factor;
    
    // write color to output
    
    float shadow = textureProj(shadow_map, SM_tex_coord);
    shadow2Color =  (ambient) + (diffuse+specular)*shadow;
    
    // perform perspective division
	//vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;
	// read depth value from shadow map
	//float depth = texture(shadow_map, tex_coords.xy).r;
	// perform depth comparison
	//float inShadow = (depth < tex_coords.z) ? 1.0 : 0.0; 
    //if in Shadow then kill diffuse and specular term
    //if(inShadow == 1.0) shadow2Color =  (ambient) + (diffuse+specular)*0.3;
    // else shadow2Color =  (ambient) + (diffuse + specular)*1.0;
	
	/*if (inShadow == 1.0) {
	shadow2Color = vec4(0.3, 0.3, 0.3, 1.0);
	}
	else {
	shadow2Color = vec4(0.5, 0.3, 0.4, 1.0);
	}*/

}
