/*
 * Simple vertex shader calculating world position, 
 * passing texture coordinates and more.
 */

#version 130              // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2

// uniform shader-parameters
uniform mat4 perspective;
uniform mat4 view;
uniform mat4 model;
uniform mat4 tex_mat;

// vertex-shader input variables
in vec4 vertex;
in vec4 color;
in vec3 normal;
in vec2 texCoord;

// vertex-shader output variables (passed to fragment-shader)
out vec4 frag_color;
out vec3 world_normal;
out vec4 world_position;
out vec2 fragTexCoord;
out vec4 SM_tex_coord; // pass on to the FS


void main()
{
	// transform normal into world space
	vec4 normal4 = vec4(normal, 0.0);
    world_normal = (model * normal4).xyz;
    
    // transform vertex to world-space
    world_position = model * vertex;
    
    // just pass color to fragment-shader
    //frag_color = color;

    // transform vertex down the pipeline
    gl_Position = perspective * view * model * vertex;
    
    // pass texture coordinate
    fragTexCoord = texCoord;

	// shadow texture coords in projected light space
	SM_tex_coord = tex_mat * model * vertex;
}